##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Python Code Interpreter

$Id: python.py 73622 2007-03-26 12:35:37Z dobe $
"""
__docformat__ = 'restructuredtext'

from StringIO import StringIO

from zope.app.interpreter.interfaces import IInterpreter
from zope.interface import implements
from zope.security.untrustedpython.interpreter import exec_src

class PythonInterpreter(object):

    implements(IInterpreter)

    def evaluate(self, code, globals):
        """See `zope.app.interfaces.IInterpreter`"""
        tmp = StringIO()
        globals['untrusted_output'] = tmp
        if isinstance(code, basestring):
            exec_src(code, globals,
                     {}, # we don't want to get local assignments saved.
                     )
        else:
            # TODO: There are no tests for this branch
            code.exec_(globals,
                       {}, # we don't want to get local assignments saved.
                       )

        return tmp.getvalue()


    def evaluateRawCode(self, code, globals):
        """See `zope.app.interfaces.IInterpreter`"""
        # Removing probable comments
        if code.strip().startswith('<!--') and code.strip().endswith('-->'):
            code = code.strip()[4:-3]

        # Prepare code.
        lines = code.splitlines()
        lines = filter(lambda l: l.strip() != '', lines)
        code = '\n'.join(lines)
        # This saves us from all indentation issues :)
        if code.startswith(' ') or code.startswith('\t'):
            code = 'if 1 == 1:\n' + code
        return self.evaluate(code, globals)


# It's a singelton for now.
PythonInterpreter = PythonInterpreter()



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces for Code Interpreters

$Id: interfaces.py 26892 2004-08-04 05:10:21Z pruggera $
"""
__docformat__ = 'restructuredtext'

from zope.interface import Interface

class IInterpreter(Interface):

    def evaluate(code, globals):
        """Evaluate the code given the global variables."""

    def evaluateRawCode(code, globals):
        """Evaluate the code given the global variables.

        However, this method does a little bit more. Sometimes (or in our case
        often) code might come from an uncontrolled environment, like a page
        template and is not properly formatted, i.e. indentation, so that some
        cleanup is necessary. This method does the cleanup before evaluating
        the code.
        """



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the workflow ZCML namespace directives.

$Id: test_pythoninterpreter.py 26819 2004-07-28 19:37:35Z jim $
"""
import unittest

from zope.app.interpreter.interfaces import IInterpreter
from zope.app.interpreter.python import PythonInterpreter
from zope.security.interfaces import ForbiddenAttribute
from zope.interface.verify import verifyObject

class PythonInterpreterTest(unittest.TestCase):

    def _check(self, code, output, raw=False):
        if raw:
            func = PythonInterpreter.evaluateRawCode
        else:
            func = PythonInterpreter.evaluate
        self.globals = {}
        self.assertEqual(func(code, self.globals), output)

    def test_verifyInterface(self):
        self.assert_(verifyObject(IInterpreter, PythonInterpreter))

    def test_simple(self):
        self._check('print "hello"', 'hello\n')
        self._check('print "hello"', 'hello\n', True)

    def test_indented(self):
        self._check('\n  print "hello"\n', 'hello\n', True)

    def test_multi_line(self):
        code = ('print "hello"\n'
                'print "world"\n')
        self._check(code, 'hello\nworld\n')
        self._check(code, 'hello\nworld\n', True)
        code = ('  print "hello"\n'
                '  print "world"\n')
        self._check(code, 'hello\nworld\n', True)

    def test_variable_assignment(self):
        code = ('x = "hello"\n'
                'print x')
        self._check(code, 'hello\n')
        self._check(code, 'hello\n', True)

    def test_local_variable_assignment(self):
        code = ('x = "hello"\n')
        self._check(code, '')
        self.assert_('x' not in self.globals.keys())

# TODO: get global statements working
# The compiler module, which we now rely on, doesn't handle global
# statements correctly.
##     def test_global_variable_assignment(self):
##         code = ('global x\n'
##                 'x = "hello"\n')
##         self._check(code, '')
##         self.assertEqual(self.globals['x'], 'hello')

    def test_wrapped_by_html_comment(self):
        self._check('<!-- print "hello" -->', 'hello\n', True)
        self._check('<!--\nprint "hello"\n -->', 'hello\n', True)
        self._check('<!--\n  print "hello"\n -->', 'hello\n', True)

    def test_forbidden_attribute(self):
        code = ('import sys\n'
                'print sys.path\n')
        self.assertRaises(ForbiddenAttribute,
                          PythonInterpreter.evaluateRawCode, code, {})

    def test_windows_newline(self):
        code = ('print "hello"\r\n'
                'print "world"\r\n')
        self._check(code, 'hello\nworld\n', True)
        

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(PythonInterpreterTest),
        ))

if __name__ == '__main__':
    unittest.main()
